@@ -186,23 +186,50 @@ class Parser(Tokenizer, ParserNumber):
                     token = self.find_prev(token_end, TokenFlags.NOT_DELIMITER)
             # Not sure why this is here, but it breaks the parser if not here
             token_end = self.find_prev(token_end, TokenFlags.VALID)
-
+        potential_type = None
+        anime_types = []
         for token in self.get_list(begin=token_begin, end=token_end):
+            if (parser_helper.is_dash_character(token.content)
+                    or token.t_category == TokenCategory.DELIMITER
+                    or token.content.upper() == "CLEAN"):
+                token_end = token
+                continue
             keyword = self.keyword_manager.find(self.keyword_manager.normalize(token.content))
             if keyword:
                 element = keyword.e_category
                 if element == ElementCategory.ANIME_TYPE:
-                    if token.content.upper() in ["ED", "NCED", "ENDING", "CLEAN ENDING",
-                                                 "OP", "NCOP", "OPENING", "CLEAN OPENING"]:
+                    anime_types.append(token)
+                    if token.content.upper() in ["ED", "NCED", "ENDING",
+                                                 "OP", "NCOP", "OPENING",
+                                                 "PV", "PREVIEW"]:
+                        clean_token = self.find_prev(token, TokenFlags.UNKNOWN)
+                        if clean_token and clean_token.content.upper() == "CLEAN":
+                            token_end = self.find_prev(clean_token)
+                            token.content = f"{clean_token.content} {token.content}"
+                            self.remove_token(clean_token)
+                        self.set_token_element(clean_token, TokenCategory.IDENTIFIER, ElementCategory.ANIME_TYPE)
+                        for anime_type in anime_types:
+                            self.set_token_element(anime_type, TokenCategory.IDENTIFIER, ElementCategory.ANIME_TYPE)
+                        if potential_type:
+                            token_end = potential_type
                         break
                     previous_token = self.find_prev(token, TokenFlags.NOT_DELIMITER)
                     if previous_token and parser_helper.is_dash_character(previous_token.content):
                         next_token = self.find_next(token, TokenFlags.NOT_DELIMITER)
                         if next_token:
                             if next_token.t_category != TokenCategory.UNKNOWN:
+                                self.set_token_element(token, TokenCategory.IDENTIFIER,
+                                                       ElementCategory.ANIME_TYPE)
                                 break
+                            if not potential_type:
+                                potential_type = token_end
                         else:
                             break
+                    if potential_type:
+                        token_end = potential_type
+            if potential_type != token_end:
+                potential_type = None
+                anime_types = []
             token_end = token
         if enclosed_title:
             token_end = self.find_prev(token_end, TokenFlags.VALID)
@@ -358,6 +385,7 @@ class Parser(Tokenizer, ParserNumber):
         - Video resolution
         - Any identifiable keywords, keywords.options["default"]
         """
+        prev = None
         for token in self.get_list(TokenFlags.UNKNOWN):
             word = token.content
             word = word.strip(' -')
@@ -377,23 +405,35 @@ class Parser(Tokenizer, ParserNumber):
                 if not ElementCategory.is_searchable(element):
                     continue
                 if not keyword.options.searchable:
+                    if element == ElementCategory.ANIME_TYPE:
+                        previous_token = self.find_prev(token, TokenFlags.NOT_DELIMITER)
+                        next_token = self.find_next(token, TokenFlags.NOT_DELIMITER)
+                        if not next_token:
+                            self.set_token_element(token, TokenCategory.IDENTIFIER, element)
+                        elif parser_helper.is_dash_character(next_token.content):
+                            self.set_token_element(token, TokenCategory.IDENTIFIER, element)
+                        elif next_token and previous_token.content == next_token.content:
+                            self.set_token_element(token, TokenCategory.IDENTIFIER, element)
                     continue
                 if not keyword.options.identifiable and not token.enclosed:
                     continue
+                if not keyword.options.valid:
+                    if not prev or not parser_helper.is_dash_character(prev.content):
+                        continue
 
-                if (element == ElementCategory.ANIME_SEASON_PREFIX
-                        and self.is_anime_season_keyword(token)):
-
-                    self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.ANIME_SEASON_PREFIX)
-
+                if element == ElementCategory.ANIME_SEASON_PREFIX:
+                    if self.is_anime_season_keyword(token):
+                        self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.ANIME_SEASON_PREFIX)
+                    continue
                 elif (element == ElementCategory.EPISODE_PREFIX
                       and keyword.options.valid):
                     if self.is_extent_keyword(token, ElementCategory.EPISODE_NUMBER, prefix=True):
                         self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.EPISODE_PREFIX)
-
+                    continue
                 elif element == ElementCategory.VOLUME_PREFIX:
                     if self.is_extent_keyword(token, ElementCategory.VOLUME_NUMBER, prefix=True):
                         self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.VOLUME_PREFIX)
+                    continue
                 elif element == ElementCategory.RELEASE_VERSION:
                     # this is probably a version number
                     token.content = word[0]
@@ -415,6 +455,8 @@ class Parser(Tokenizer, ParserNumber):
                     self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.OTHER)
                     continue
                 elif element == ElementCategory.RELEASE_INFORMATION:
+                    if prev and prev.content.lower() in ["the"]:
+                        continue
                     self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.RELEASE_INFORMATION)
                     continue
                 elif element == ElementCategory.SOURCE:
@@ -426,12 +468,9 @@ class Parser(Tokenizer, ParserNumber):
                 elif element == ElementCategory.VIDEO_TERM:
                     self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.VIDEO_TERM)
                     continue
-
                 # last resort to avoid false positives
                 if token.enclosed:
                     self.set_token_element(token, TokenCategory.IDENTIFIER, element)
-                else:
-                    continue
             else:
                 # we could have multiple valid crc and resolution keywords.
                 if parser_helper.is_crc32(word):
@@ -442,6 +481,7 @@ class Parser(Tokenizer, ParserNumber):
             if element != ElementCategory.UNKNOWN:
                 if keyword is None or keyword.options.identifiable:
                     self.set_token_element(token, TokenCategory.IDENTIFIER, element)
+            prev = token
 
     def search_for_release_group(self) -> None:
         """
@@ -501,8 +541,10 @@ class Parser(Tokenizer, ParserNumber):
 
             tokens = self.get_list(begin=token_start, end=token_episode)
 
+            multi = False
             for token in tokens:
                 if token.e_category == ElementCategory.RANGE_SEPARATOR:
+                    multi = True
                     break
                 if token.e_category == ElementCategory.EPISODE_NUMBER:
                     if float(token_start_episode.content) == float(token.content):
@@ -516,12 +558,110 @@ class Parser(Tokenizer, ParserNumber):
                     self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.EPISODE_NUMBER_ALT)
                     break
 
+            not_enclosed_tokens = [token_start_episode]
+            enclosed_tokens = []
+            for token in tokens:
+                if token.e_category in [ElementCategory.EPISODE_NUMBER, ElementCategory.EPISODE_NUMBER_ALT]:
+                    if token.enclosed:
+                        enclosed_tokens.append(token)
+                    else:
+                        not_enclosed_tokens.append(token)
+
+            if multi:
+                if len(enclosed_tokens) == len(not_enclosed_tokens) == 2:
+
+                    if self.options['eps_lower_than_alt']:
+                        for idx, token in enumerate(enclosed_tokens):
+                            if float(enclosed_tokens[idx].content) > float(not_enclosed_tokens[idx].content):
+                                self.set_token_element(token, TokenCategory.IDENTIFIER,
+                                                       ElementCategory.EPISODE_NUMBER_ALT)
+                            else:
+                                self.set_token_element(not_enclosed_tokens[idx], TokenCategory.IDENTIFIER,
+                                                       ElementCategory.EPISODE_NUMBER_ALT)
+                    else:
+                        for token in enclosed_tokens:
+                            self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.EPISODE_NUMBER_ALT)
+
         # Fill the unknown tokens with the keywords
         for token in self.get_list(TokenFlags.UNKNOWN):
             keyword = self.keyword_manager.find(self.keyword_manager.normalize(token.content))
             if keyword:
                 self.set_token_element(token, TokenCategory.IDENTIFIER, keyword.e_category)
 
+        tokens = self.get_list(elements=ElementCategory.EPISODE_TITLE)
+        real_token = []
+        for token in tokens:
+            if not token.content.strip(" " + parser_helper.DASHES):
+                continue
+            if token.t_category in [TokenCategory.DELIMITER, TokenCategory.BRACKET]:
+                continue
+            real_token.append(token)
+
+        if len(real_token) == 1:
+            keyword = self.keyword_manager.find(self.keyword_manager.normalize(real_token[0].content))
+            if keyword:
+                self.set_token_element(real_token[0], TokenCategory.IDENTIFIER, keyword.e_category)
+        elif len(real_token) == 2:
+            keyword = self.keyword_manager.find(self.keyword_manager.normalize(real_token[1].content))
+            if keyword and keyword.e_category == ElementCategory.ANIME_TYPE:
+                if (real_token[0].content.upper() == "CLEAN"
+                        and real_token[1].content.upper() in ["ED", "NCED", "ENDING",
+                                                              "OP", "NCOP", "OPENING"]):
+                    self.set_token_element(real_token[0], TokenCategory.IDENTIFIER, keyword.e_category)
+                    real_token[0].content += " " + real_token[1].content
+                    self.remove_token(real_token[1])
+        elif real_token:
+            keyword = self.keyword_manager.find(self.keyword_manager.normalize(real_token[-1].content))
+            if keyword and keyword.e_category == ElementCategory.ANIME_TYPE:
+                self.set_token_element(real_token[-1], TokenCategory.IDENTIFIER, keyword.e_category)
+
+        # Check for unknown tokens between Season and Episode
+        token_season = self.find_next(None, flags=TokenFlags.IDENTIFIER,
+                                      element=[
+                                          ElementCategory.ANIME_SEASON_PREFIX,
+                                          ElementCategory.ANIME_SEASON,
+                                      ])
+        token_type = self.find_next(None, flags=TokenFlags.IDENTIFIER,
+                                    element=[
+                                        ElementCategory.ANIME_TYPE,
+                                    ])
+        token_number = self.find_next(None, flags=TokenFlags.IDENTIFIER,
+                                      element=[
+                                          ElementCategory.EPISODE_NUMBER,
+                                      ])
+        if token_season:
+            token_end = token_type or token_number
+            if token_end:
+                actually_a_title = False
+                for token in self.get_list(TokenFlags.UNKNOWN, token_season, token_end):
+                    if parser_helper.is_dash_character(token.content) or not token.content:
+                        continue
+                    actually_a_title = True
+                if actually_a_title:
+                    # set token from end of anime title until season token to be anime title
+                    anime_title = self.find_prev(token_season, flags=TokenFlags.IDENTIFIER)
+                    for token in self.get_list(None, anime_title, token_end):
+                        if token.e_category in [token_end.e_category]:
+                            break
+                        if token.e_category == ElementCategory.ANIME_SEASON:
+                            token.content = str(int(token.content))
+                        self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.ANIME_TITLE)
+
+        if token_type and token_number:
+            actually_a_title = False
+            for token in self.get_list(TokenFlags.UNKNOWN, token_type, token_number):
+                if parser_helper.is_dash_character(token.content) or not token.content:
+                    continue
+                actually_a_title = True
+            if actually_a_title:
+                # set token from end of anime title until season token to be anime title
+                anime_title = self.find_prev(token_type, flags=TokenFlags.IDENTIFIER)
+                for token in self.get_list(None, anime_title, token_number):
+                    if (token.e_category in [token_number.e_category, ElementCategory.ANIME_TYPE]
+                            and token != token_type):
+                        break
+                    self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.ANIME_TITLE)
+
         # make sure there's no anime type in the episode title
         tokens = self.get_list(elements=ElementCategory.EPISODE_TITLE)
         is_other = False
@@ -534,20 +674,7 @@ class Parser(Tokenizer, ParserNumber):
                 elif is_other:
                     self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.OTHER)
 
-        tokens = self.get_list(elements=ElementCategory.EPISODE_TITLE)
-        real_token = []
-        for token in tokens:
-            if parser_helper.is_dash_character(token.content):
-                continue
-            if token.t_category in [TokenCategory.DELIMITER, TokenCategory.BRACKET]:
-                continue
-            real_token.append(token)
-
-        if len(real_token) == 1:
-            keyword = self.keyword_manager.find(self.keyword_manager.normalize(real_token[0].content))
-            if keyword:
-                self.set_token_element(real_token[0], TokenCategory.IDENTIFIER, keyword.e_category)
-
+        # Anime type `Movie` are invalid if the last token is `the`
         token = self.find_prev(None, flags=TokenFlags.IDENTIFIER, element=ElementCategory.ANIME_TITLE)
         if token:
             if token.content.lower() == "the":
