@@ -3,6 +3,7 @@ from typing import List
 
 from aniparse import parser_helper
 from aniparse.element import ElementCategory
+from aniparse.keyword import Keyword
 from aniparse.token import Token, TokenCategory, TokenFlags, Tokens
 
 ANIME_YEAR_MIN = 1900
@@ -11,6 +12,31 @@ ANIME_YEAR_MAX = 2100
 
 # noinspection DuplicatedCode
 class EpisodePattern(Tokens):
+    _re_japanese_counter = re.compile(r"^(第)?(\d+)"  # prefix
+                                      r"(?:([" + parser_helper.ESCAPED_DASHES +
+                                      r" ~&+]?)"  # prefix separator
+                                      r"(\d+))?"  # number
+                                      r"([話弾])$",  # prefix
+                                      flags=re.IGNORECASE)
+    _re_number_sign = re.compile(r"^(#)(\d+(?:\.\d+)?)"  # sign and number
+                                 r"(?:([" + parser_helper.ESCAPED_DASHES +
+                                 r" ~&+]?)"  # separator
+                                 r"(\d+(?:\.\d+)?))?"  # 2nd number
+                                 r"(?:(v)(\d+))?$",  # version
+                                 flags=re.IGNORECASE)
+    _re_season_episode = re.compile(r"^(?:(\d+)|(s)(\d+)"  # Season prefix or number
+                                    r"(?:([" + parser_helper.ESCAPED_DASHES +
+                                    r" ~&+])"  # separator
+                                    r"(s)?(\d+))?)"  # 2nd season prefix or number
+                                    # Season "1", "s01", "s01-02", "s01~02"
+                                    r"(?:(x)|([" + parser_helper.ESCAPED_DASHES +
+                                    r" ._\-x:])?(e))"  # Season-Episode separator
+                                    r"(\d+(?:\.\d+)?)(?:(v)(\d+))?"  # Episode "01", "01v2"
+                                    r"(?:([" + parser_helper.ESCAPED_DASHES +
+                                    r"~&+])(e)?"  # Episode range separator
+                                    r"(\d+(?:\.\d+)?)(?:(v)(\d+))?)?$",  # Episode "01", "01v2"
+                                    flags=re.IGNORECASE)
+
     def is_match_japanese_counter_pattern(self, token: Token, word: str) -> bool:
         """
         Checking for japanese style counter pattern
@@ -19,9 +45,7 @@ class EpisodePattern(Tokens):
         pattern
         <japanese counter><episode number>[<range_separator><episode number>]
         """
-        pattern = (r"^(第)?(\d+)(?:([\ \~\&\+ " + parser_helper.ESCAPED_DASHES +
-                   r"]?)(\d+))?(話|弾)$")
-        match = re.match(pattern, word, flags=re.IGNORECASE)
+        match = self._re_japanese_counter.match(word)
         if match:
             if match.group(5):
                 self.insert_tokens(token, match.group(5), TokenCategory.IDENTIFIER, ElementCategory.UNKNOWN)
@@ -46,9 +70,7 @@ class EpisodePattern(Tokens):
         pattern
         <number sign><episode number>[<range_separator><episode number>][<version>]
         """
-        pattern = (r"^(#)(\d+(?:\.\d+)?)(?:([\ " + parser_helper.ESCAPED_DASHES +
-                   r"\~\&\+]?)(\d+(?:\.\d+)?))?(?:(v)(\d+))?$")
-        match = re.match(pattern, word, flags=re.IGNORECASE)
+        match = self._re_number_sign.match(word)
         if match:
             if match.group(6):
                 self.insert_tokens(token, match.group(6), TokenCategory.IDENTIFIER, ElementCategory.RELEASE_VERSION)
@@ -82,18 +104,7 @@ class EpisodePattern(Tokens):
         <episode_number>[v<release_version>][<range_separator>[episode_prefix]<episode_number>[v<release_version>]]
         """
 
-        pattern = (r"^(?:(\d+)|([s])(\d+)(?:([" + parser_helper.ESCAPED_DASHES +
-                   r"\~\&\+])(s)?(\d+))?)"  # Season "1", "s01", "s01-02", "s01~02"
-                   r"(?:(x)|([\ \._\-x:" + parser_helper.ESCAPED_DASHES +
-                   r"])?(e))"  # Season-Episode separator
-                   r"(\d+(?:\.\d+)?)(?:([v])(\d+))?"  # Episode "01", "01v2"
-                   r"(?:([" + parser_helper.ESCAPED_DASHES +
-                   r"\~\&\+])(e)?"  # Episode range separator
-                   r"(\d+(?:\.\d+)?)(?:([v])(\d+))?)?"  # Episode "01", "01v2"
-                   # Optional, first part of the episode title
-                   r"(?:([" + parser_helper.ESCAPED_DASHES +
-                   r"])(.*))?$")
-        match = re.match(pattern, word, flags=re.IGNORECASE)
+        match = self._re_season_episode.match(word)
 
         if match:
             # check for lower bound < upper bound
@@ -113,10 +124,6 @@ class EpisodePattern(Tokens):
             except ValueError:
                 pass
             # in reverse order to keep the order of tokens
-            if match.group(19):
-                self.insert_tokens(token, match.group(19), TokenCategory.UNKNOWN, ElementCategory.UNKNOWN)
-            if match.group(18):
-                self.insert_tokens(token, match.group(18), TokenCategory.UNKNOWN, ElementCategory.UNKNOWN)
             if match.group(17):
                 self.insert_tokens(token, match.group(17), TokenCategory.IDENTIFIER, ElementCategory.RELEASE_VERSION)
             if match.group(16):
@@ -167,6 +174,14 @@ class Pattern(EpisodePattern, VolumePattern):
     """
     Main pattern that apply for Episode, Volume, and Season.
     """
+    _re_match_multi_number = re.compile(r"(\d+(?:\.\d+)?)(?:(v)(\d+))?"  # Episode number, release version
+                                        r"([" + parser_helper.ESCAPED_DASHES +
+                                        r" ~&+])"  # Range separator
+                                        r"(\d+(?:\.\d+)?)(?:(v)(\d+))?$",  # Episode number, release version
+                                        flags=re.IGNORECASE)
+    _re_match_single_number = re.compile((r'^(\d+(?:\.\d+)?)'  # Episode number
+                                          r'(?:(v)(\d+))?$'),  # Release version
+                                         flags=re.IGNORECASE)
 
     def is_match_multi_numbers_pattern(self, token: Token, word: str, category: ElementCategory) -> bool:
         """
@@ -174,9 +189,7 @@ class Pattern(EpisodePattern, VolumePattern):
         e.g. "01-02", "01-02v2", "01v2-02v2", "01v2-02", "04.5-05.5", "04.5-05.5v2", "04.5v2-05.5v2", "04.5v2-05.5"
         pattern = <episode_number>[v<release_version>]<range_separator><episode_number>[v<release_version>]
         """
-        pattern = (r"(\d+(?:\.\d+)?)(?:([v])(\d+))?([" + parser_helper.ESCAPED_DASHES +
-                   r"\~\&\+])(\d+(?:\.\d+)?)(?:([v])(\d+))?$")
-        match = re.match(pattern, word, flags=re.IGNORECASE)
+        match = self._re_match_multi_number.match(word)
         if match:
             lower_bound = match.group(1)
             upper_bound = match.group(5)
@@ -210,7 +223,7 @@ class Pattern(EpisodePattern, VolumePattern):
         """
         Check if the word have a valid numbered pattern.
         """
-        if parser_helper.is_number(word) and not prefix:
+        if parser_helper.is_potential_number(word) and not prefix:
             # If the word only a number and the prefix is not set, then try to check the previous token
             previous_token = self.find_prev(token, TokenFlags.IDENTIFIER | TokenFlags.UNKNOWN)
             if previous_token.content.lower() in ["part"]:
@@ -303,8 +316,7 @@ class Pattern(EpisodePattern, VolumePattern):
         e.g. "01", "01v2", "07.5", "07.5v2"
         pattern = <episode_number>[v<release_version>]
         """
-        pattern = '^(\\d+(?:\\.\\d+)?)(?:([v])(\\d+))?$'
-        match = re.match(pattern, word, flags=re.IGNORECASE)
+        match = self._re_match_single_number.match(word)
         if match:
             # in reverse order to keep the order of tokens
             if match.group(3):
@@ -319,6 +331,8 @@ class Pattern(EpisodePattern, VolumePattern):
 
 
 class ParserNumber(Pattern):
+    _re_search_buggy_dash = re.compile(r"(" + "|".join(parser_helper.DASHES) + r")",
+                                       flags=re.IGNORECASE)
 
     def is_extent_keyword(self, token: Token, category: ElementCategory, prefix: bool) -> bool:
         next_token = self.find_next(token, TokenFlags.NOT_DELIMITER)
@@ -349,6 +363,34 @@ class ParserNumber(Pattern):
                 category = ElementCategory.VOLUME_NUMBER
             elif keyword.e_category == ElementCategory.ANIME_SEASON_PREFIX:
                 category = ElementCategory.ANIME_SEASON
+                additional_prefix = parser_helper.find_non_number_in_string(number)
+                if additional_prefix:
+                    left_over = number[additional_prefix:]
+                    new_number_begin = parser_helper.find_number_in_string(left_over)
+                    key = left_over[:new_number_begin]
+                    lo_keyword = self.keyword_manager.find(
+                        self.keyword_manager.normalize(key)
+                    )
+                    if (lo_keyword
+                            and lo_keyword.e_category == ElementCategory.ANIME_TYPE):
+                        number = number[:additional_prefix]
+                    else:
+                        if key.upper() == "S":
+                            new_token = Token(prefix,
+                                              TokenCategory.IDENTIFIER,
+                                              ElementCategory.ANIME_SEASON_PREFIX,
+                                              token.enclosed)
+                            self.insert_before(token, new_token)
+                            new_token = Token(number[:additional_prefix],
+                                              TokenCategory.IDENTIFIER,
+                                              ElementCategory.ANIME_SEASON,
+                                              token.enclosed)
+                            self.insert_before(token, new_token)
+                            keyword = Keyword(ElementCategory.ANIME_TYPE, keyword.options)
+                            category = ElementCategory.EPISODE_NUMBER
+                            prefix = key
+                            number = left_over[new_number_begin:]
+                        del left_over
             elif keyword.e_category == ElementCategory.ANIME_TYPE:
                 category = ElementCategory.EPISODE_NUMBER
             else:
@@ -357,7 +399,28 @@ class ParserNumber(Pattern):
             if self.is_match_number_patterns(token, number, category, True):
                 new_token = Token(prefix, TokenCategory.IDENTIFIER, keyword.e_category, token.enclosed)
                 self.insert_before(token, new_token)
+                try:
+                    left_token = Token(left_over, TokenCategory.UNKNOWN, ElementCategory.UNKNOWN, token.enclosed)
+                    self.insert_after(token, left_token)
+                    self.is_number_comes_after_prefix(left_token)
+                except NameError:
+                    pass
                 return True
+        else:
+            number = token.content[number_begin:]
+            if prefix and prefix[-1].lower() == "v" and number.isdigit():
+                keyword = self.keyword_manager.find(
+                    self.keyword_manager.normalize(prefix[:-1])
+                )
+                if keyword and keyword.e_category in [ElementCategory.EPISODE_PREFIX, ElementCategory.VOLUME_PREFIX,
+                                                      ElementCategory.ANIME_SEASON_PREFIX, ElementCategory.ANIME_TYPE]:
+                    token.content = prefix[:-1]
+                    self.set_token_element(token, TokenCategory.IDENTIFIER, keyword.e_category)
+                    new_token = Token(prefix[-1], TokenCategory.INVALID, ElementCategory.UNKNOWN, token.enclosed)
+                    self.insert_after(token, new_token)
+                    new_token = Token(number, TokenCategory.IDENTIFIER, ElementCategory.RELEASE_VERSION, token.enclosed)
+                    self.insert_after(token, new_token)
+                    return True
         return False
 
     def is_number_comes_before_another_number(self, token: Token) -> bool:
@@ -391,19 +454,61 @@ class ParserNumber(Pattern):
         :return:
         """
         for token in tokens:
-            match = re.match(r"(.*)(" + "|".join(parser_helper.DASHES) +
-                             r")(.*?)(\d.*)", token.content, re.IGNORECASE)
-            if match:
-                token.content = match.group(3) + match.group(4)
-                index = self.get_index(token)
-                if self.is_number_comes_after_prefix(token):
-                    new_delimiter = Token(match.group(2), TokenCategory.DELIMITER, ElementCategory.DELIMITER,
-                                          token.enclosed)
-                    self.insert(index, new_delimiter)
-                    new_token = Token(match.group(1), TokenCategory.UNKNOWN, ElementCategory.UNKNOWN, token.enclosed)
-                    self.insert(index, new_token)
+            splitted = self._re_search_buggy_dash.split(token.content)
+            placeholder = []
+            new_token = Token("", token.t_category, token.e_category, token.enclosed)
+            potential_tokens = []
+            for text in splitted:
+                if parser_helper.find_number_in_string(text) is not None:
+                    if new_token.content:
+                        # first and last dashes put on separate token
+                        stripped_text = new_token.content.strip(parser_helper.DASHES)
+                        dashes = new_token.content.split(stripped_text)
+                        if dashes[0]:
+                            placeholder.append(Token(dashes[0], TokenCategory.INVALID,
+                                                     ElementCategory.UNKNOWN, token.enclosed))
+                        new_token.content = stripped_text
+                        placeholder.append(new_token)
+                        if dashes[1]:
+                            placeholder.append(Token(dashes[1], TokenCategory.INVALID,
+                                                     ElementCategory.UNKNOWN, token.enclosed))
+
+                        new_token = Token("", token.t_category, token.e_category, token.enclosed)
+                    number_token = Token(text, token.t_category, token.e_category, token.enclosed)
+                    placeholder.append(number_token)
+                    potential_tokens.append(number_token)
                 else:
-                    token.content = match.group(1) + match.group(2) + match.group(3) + match.group(4)
+                    new_token.content += text
+
+            if new_token.content:
+                stripped_text = new_token.content.strip(parser_helper.DASHES)
+                dashes = new_token.content.split(stripped_text)
+                if dashes[0]:
+                    placeholder.append(Token(dashes[0], TokenCategory.INVALID,
+                                             ElementCategory.UNKNOWN, token.enclosed))
+                new_token.content = stripped_text
+                placeholder.append(new_token)
+                if dashes[1]:
+                    placeholder.append(Token(dashes[1], TokenCategory.INVALID,
+                                             ElementCategory.UNKNOWN, token.enclosed))
+
+            if placeholder:
+                placeholder.reverse()
+                for i_token in placeholder:
+                    self.insert_after(token, i_token)
+                self.remove_token(token)
+
+                is_found = False
+                for potential_token in potential_tokens:
+                    if self.is_number_comes_after_prefix(potential_token):
+                        is_found = True
+                    elif self.search_for_last_number([potential_token]):
+                        is_found = True
+
+                if not is_found:
+                    self.insert_after(placeholder[-1], token)
+                    for i_token in placeholder:
+                        self.remove_token(i_token)
         return
 
     def search_for_equivalent_numbers(self, tokens: List[Token]) -> bool:
@@ -479,16 +584,30 @@ class ParserNumber(Pattern):
 
             # Ignore if the previous token is "Movie" or "Part"
             previous_token = self.find_prev(token, TokenFlags.NOT_DELIMITER)
-            if previous_token and str(previous_token.content).lower() in ["movie", "part"]:
+            if previous_token and str(previous_token.content).lower() in ["part"]:
                 continue
             previous_token = self.find_prev(token, TokenFlags.UNKNOWN | TokenFlags.BRACKET)
-            if not previous_token:
+            if not previous_token or previous_token == token:  # , because previous_token most likely same as token
                 continue
             if not parser_helper.is_dash_character(previous_token.content):
+                next_token = self.find_next(token, TokenFlags.NOT_DELIMITER)
+                if next_token and parser_helper.is_dash_character(next_token.content):
+                    try:
+                        f_text = float(token.content)
+                        if f_text.is_integer():
+                            if str(int(f_text)) != token.content:
+                                self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.EPISODE_NUMBER)
+                                is_found = True
+                        elif str(f_text) != token.content:
+                            self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.EPISODE_NUMBER)
+                            is_found = True
+                    except ValueError:
+                        pass
+
                 continue
             if token.content.isdigit():
                 self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.EPISODE_NUMBER)
-                return True
+                is_found = True
         return is_found
 
     def search_for_number_patterns(self, tokens: List[Token]) -> bool:
@@ -511,6 +630,8 @@ class ParserNumber(Pattern):
                 # if previous token is a dash, it is most likely an episode number
                 prefix = False
                 previous_token = self.find_prev(token, TokenFlags.NOT_DELIMITER)
+                keyword = self.keyword_manager.find(
+                    self.keyword_manager.normalize(previous_token.content))
                 prev_is_dash = parser_helper.is_dash_character(previous_token.content)
                 if prev_is_dash:
                     prefix = True
@@ -527,10 +648,30 @@ class ParserNumber(Pattern):
                                 or (previous_token.t_category == next_token.t_category == TokenCategory.BRACKET
                                     and self.counter.get(ElementCategory.EPISODE_NUMBER))):
                             prefix = True
-                    else:  # The number are at the end of the string
-                        prefix = True
+                    else:
+                        # 02 is most likely an episode number
+                        # but 2 is not guaranteed
+                        try:
+                            is_number = float(token.content)
+                            if is_number.is_integer():
+                                if str(int(is_number)) != token.content:
+                                    prefix = True
+                            else:
+                                if str(is_number) != token.content:
+                                    prefix = True
+                        except ValueError:
+                            # in case it 04v2, then it most likely an episode number
+                            prefix = True
+
+                    if not prefix:
+                        if keyword and keyword.e_category == ElementCategory.ANIME_TYPE:
+                            prefix = True
+
                 if self.is_match_number_patterns(token, token.content, ElementCategory.EPISODE_NUMBER, prefix):
                     is_found = True
+                    if prefix:
+                        if keyword and keyword.e_category == ElementCategory.ANIME_TYPE:
+                            self.set_token_element(previous_token, TokenCategory.IDENTIFIER, ElementCategory.ANIME_TYPE)
         return is_found
 
     def search_for_seperated_numbers(self, tokens: List[Token]) -> bool:
