@@ -522,6 +522,21 @@ table = [
         "source": "BD",
         "video_resolution": "1080p"
     },
+    {
+        "anime_title": "Toradora! asdwd",
+        "episode_number": 3,
+        "episode_title": "Your-Song",
+        "file_extension": "mkv",
+        "file_name": "Toradora! asdwd-03-Your-Song.mkv"
+    },
+    {
+        "anime_title": "Toradora",
+        "episode_number": 3,
+        "episode_prefix": "E",
+        "episode_title": "Your-Song",
+        "file_extension": "mkv",
+        "file_name": "Toradora-E03-Your-Song.mkv"
+    },
     {
         "anime_season": 1,
         "anime_season_prefix": "S",
@@ -644,6 +659,19 @@ table = [
         "video_resolution": "1080p",
         "video_term": "x264"
     },
+    {
+        "anime_title": "Shingeki no Kyojin (The Final Season)",
+        "audio_term": "Dual Audio",
+        "episode_number": 13,
+        "episode_number_alt": 72,
+        "file_extension": "mkv",
+        "file_name": "[AnimeRG] Shingeki no Kyojin (The Final Season) - 13 "
+                     "[1080p 10bit Dual Audio] (Attack on Titan - 72).mkv",
+        "other": ["Attack", "on", "Titan"],
+        "release_group": "AnimeRG",
+        "video_resolution": "1080p",
+        "video_term": "10bit"
+    },
     {
         "anime_title": "Ushio to Tora (TV)",
         "episode_number": 2,
@@ -653,6 +681,17 @@ table = [
         "release_group": "AnimeRG",
         "video_resolution": "720p"
     },
+    {
+        "anime_title": "Witch Craft Works",
+        "anime_type": "ED",
+        "file_extension": "mkv",
+        "file_name": "[AnimeRG] Witch Craft Works - EDv2 [1080p] [x265] [pseudo].mkv",
+        "other": "pseudo",
+        "release_group": "AnimeRG",
+        "release_version": 2,
+        "video_resolution": "1080p",
+        "video_term": "x265"
+    },
     {
         "anime_title": "Mangaka-san to Assistant-san to the Animation",
         "audio_term": "AAC",
@@ -1025,8 +1064,8 @@ table = [
         "release_group": "FFF"
     },
     {
-        "anime_title": "Love Live! The School Idol Movie",
-        "anime_type": "PV",
+        "anime_title": "Love Live! The School Idol",
+        "anime_type": ["Movie", "PV"],
         "file_checksum": "D1A15D2C",
         "file_extension": "mkv",
         "file_name": "[FFF] Love Live! The School Idol Movie - PV [D1A15D2C].mkv",
@@ -1095,6 +1134,13 @@ table = [
         "file_name": "[GrimRipper] Koi Kaze [Dual Audio] Ep01 (c13cefe0).mkv",
         "release_group": "GrimRipper"
     },
+    {
+        "file_name": "[Hakata Ramen] Cells At Work! Special.mkv",
+        "file_extension": "mkv",
+        "release_group": "Hakata Ramen",
+        "anime_title": "Cells At Work!",
+        "anime_type": "Special"
+    },
     {
         "anime_title": "Evangelion 3.0 You Can (Not) Redo",
         "file_extension": "mp4",
@@ -1278,6 +1324,124 @@ table = [
         "release_group": "Judas",
         "release_version": 2
     },
+    {
+        "anime_season": 1,
+        "anime_season_prefix": "S",
+        "anime_title": "Amanchu!",
+        "anime_type": "Ending",
+        "file_extension": "mkv",
+        "file_name": "[Judas] Amanchu! S1 - Ending.mkv",
+        "release_group": "Judas"
+    },
+    {
+        "anime_title": "Black Clover Jump Special Anime Festa 2018",
+        "file_name": "[Judas] Black Clover - Jump Special Anime Festa 2018.mkv",
+        "file_extension": "mkv",
+        "release_group": "Judas"
+    },
+    {
+        "anime_title": "Black Clover",
+        "anime_type": "NCOP",
+        "episode_number": 10,
+        "episode_part": "b",
+        "episode_title": "Black Catcher",
+        "file_name": "[Judas] Black Clover - NCOP 10b - Black Catcher.mkv",
+        "file_extension": "mkv",
+        "release_group": "Judas",
+    },
+    {
+        "anime_title": "Boruto",
+        "anime_type": "OVA",
+        "episode_title": "The Day Naruto Became Hokage",
+        "file_name": "[Judas] Boruto - OVA - The Day Naruto Became Hokage.mkv",
+        "file_extension": "mkv",
+        "release_group": "Judas"
+    },
+    {
+        "anime_title": "Card Captor Sakura The Movie",
+        "episode_number": 2,
+        "episode_title": "The Sealed Card",
+        "file_extension": "mkv",
+        "file_name": "[Judas] Card Captor Sakura - The Movie 2 - The Sealed Card.mkv",
+        "release_group": "Judas"
+    },
+    {
+        "anime_title": "Code Geass",
+        "anime_type": "Movie",
+        "episode_number": 2,
+        "file_extension": "mkv",
+        "file_name": "[Judas] Code Geass Movie 02.mkv",
+        "release_group": "Judas"
+    },
+    {
+        "anime_title": "Code Geass",
+        "anime_type": "Movie",
+        "episode_number": 2,
+        "episode_title": "Lelouch of the Rebellion Transgression",
+        "file_extension": "mkv",
+        "file_name": "[Judas] Code Geass Movie 02 - Lelouch of the Rebellion - Transgression.mkv",
+        "release_group": "Judas"
+    },
+    {
+        "anime_season": 1,
+        "anime_season_prefix": "S",
+        "anime_title": "Gintama",
+        "episode_number": [1, 2],
+        "episode_number_alt": [1, 2],
+        "episode_prefix": ["E", "E"],
+        "file_extension": "mkv",
+        "file_name": "[Judas] Gintama - S01E01-E02 (001-002).mkv",
+        "release_group": "Judas"
+    },
+    {
+        "anime_season": 2,
+        "anime_season_prefix": "S",
+        "anime_title": "Gintama",
+        "episode_number": [1, 2],
+        "episode_number_alt": [25, 26],
+        "episode_prefix": ["E", "E"],
+        "file_extension": "mkv",
+        "file_name": "[Judas] Gintama - S02E01-E02 (025-026).mkv",
+        "release_group": "Judas"
+    },
+    {
+        "anime_title": "Granblue Fantasy",
+        "anime_type": "Clean Ending",
+        "file_extension": "mkv",
+        "file_name": "[Judas] Granblue Fantasy - Clean Ending.mkv",
+        "release_group": "Judas"
+    },
+    {
+        "anime_season": 1,
+        "anime_season_prefix": "S",
+        "anime_title": "Grisaia no Kajitsu",
+        "anime_type": "S",
+        "episode_number": 2,
+        "file_extension": "mkv",
+        "file_name": "[Judas] Grisaia no Kajitsu - S01S02.mkv",
+        "release_group": "Judas"
+    },
+    {
+        "anime_title": "Initial D Extra Stage",
+        "episode_number": 2,
+        "episode_title": "Sentimental White",
+        "file_extension": "mkv",
+        "file_name": "[Judas] Initial D - Extra Stage 02 - Sentimental White.mkv",
+        "release_group": "Judas"
+    },
+    {
+        "anime_title": "Initial D Extra Stage 2 Sentimental White",
+        "file_extension": "mkv",
+        "file_name": "[Judas] Initial D - Extra Stage 2 - Sentimental White.mkv",
+        "release_group": "Judas"
+    },
+    {
+        "anime_title": "Nogizaka Haruka no Himitsu",
+        "anime_type": ["OVA", "NCOP"],
+        "file_extension": "mkv",
+        "file_name": "[Judas] Nogizaka Haruka no Himitsu - OVA NCOP.mkv",
+        "release_group": "Judas"
+    },
     {
         "anime_title": "One Piece",
         "episode_number": 1009,
@@ -1286,6 +1450,16 @@ table = [
         "release_group": "Judas",
         "release_version": 2
     },
+    {
+        "anime_season": 1,
+        "anime_season_prefix": "S",
+        "anime_title": "One Punch Man",
+        "anime_type": "OVA",
+        "episode_number": 2,
+        "file_extension": "mkv",
+        "file_name": "[Judas] One Punch Man - S01OVA02.mkv",
+        "release_group": "Judas"
+    },
     {
         "anime_season": 1,
         "anime_season_prefix": "Season",
@@ -1544,7 +1718,8 @@ table = [
         "video_term": "h264"
     },
     {
-        "anime_title": "AIKa ZERO OVA",
+        "anime_title": "AIKa ZERO",
+        "anime_type": "OVA",
         "audio_term": "Flac",
         "episode_number": 1,
         "file_checksum": "6730D40A",
@@ -1780,6 +1955,27 @@ table = [
         "source": "BD",
         "video_resolution": "1080p"
     },
+    {
+        "anime_title": "Nanatsu no Taizai S3 Kamigami no Gekirin",
+        "episode_number": 21,
+        "file_extension": "mkv",
+        "file_name": "[zza] Nanatsu no Taizai - S03 - Kamigami no Gekirin - 21 [1080p.x265].mkv",
+        "release_group": "zza",
+        "video_resolution": "1080p",
+        "video_term": "x265"
+    },
+    {
+        "anime_season": 3,
+        "anime_season_prefix": "S",
+        "anime_title": "Nanatsu no Taizai",
+        "anime_type": "OVA",
+        "episode_number": 21,
+        "file_extension": "mkv",
+        "file_name": "[zza] Nanatsu no Taizai - S03 - OVA - 21 [1080p.x265].mkv",
+        "release_group": "zza",
+        "video_resolution": "1080p",
+        "video_term": "x265"
+    },
     {
         "anime_title": "Shingeki no Kyojin",
         "audio_term": "AAC",
